import java.awt.*;
import objectdraw.*;

/*
 * A very basic recursive data structure to represent
 * a Scribble, augmented with contains, move, and isEmpty
 * methods.  Now with setColor!
 *
 * Jim Teresco, CSC 252, The College of Saint Rose, Fall 2013
 * Based on similar example from Williams College CS 134.
 * 
 * $Id: Scribble.java 2249 2013-11-07 16:20:52Z terescoj $
 */

// A class to represent a scribble
public class Scribble {
    private Line first;           // an edge line of the scribble
    private Scribble rest;        // the rest of the scribble

    // Our constructor here doesn't actually do any of the
    // drawing - it takes a line that someone else created and
    // which is supposed to be part of the Scribble, and
    // a reference to another Scribble which represents
    // any previous Scribble.
    public Scribble(Line segment, Scribble theRest) {
        first = segment;
        rest = theRest;
    }

    // returns true iff the scribble contains the point
    // Note that things are slightly more complex than some
    // of our other recursive accessors as there are three
    // possibilities: 1) this Scribble is empty, in which
    // case it cannot contain the point, 2) first contains
    // the point, in which case the whole Scribble contains
    // the point, or 3) the answer depends on whether rest
    // (itself a Scribble, hence the recursive call) contains
    // the point
    public boolean contains(Location point) {
        if (isEmpty()) {
            return false;
        } else if (first.contains(point)) {
            return true;
        } else {
            return rest.contains(point);
        }
    }



    // the scribble is moved xOffset in the x direction
    //    and yOffset in the y direction
    public void move(double xOffset, double yOffset) {
        if (!isEmpty()) {
            first.move(xOffset, yOffset);
            rest.move(xOffset, yOffset);
        }
    }

    // returns true iff the list is empty
    public boolean isEmpty() {
        return (first == null);
    }
    
    // set the color of this Scribble
    public void setColor(Color c) {
        
        if (!isEmpty()) {
            first.setColor(c);
            rest.setColor(c);
        }
    }
}


